@@ -30,6 +30,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
+#include <locale.h>
 
 #ifdef HAVE_GLIB
 #include <glib.h>
@@ -37,19 +38,20 @@
 #include <mdbfakeglib.h>
 #endif
 
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV)
 #include <iconv.h>
-#else
-#ifdef HAVE_XLOCALE_H
+#elif defined(HAVE_XLOCALE_H)
 #include <xlocale.h>
 #endif
-#include <locale.h>
-#endif
 
 #ifdef _WIN32
 #include <io.h>
 #endif
 
+/** \addtogroup mdbtools
+ *  @{
+ */
+
 #define MDB_DEBUG 0
 
 #define MDB_PGSIZE 4096
@@ -208,9 +210,10 @@ typedef struct mdbsargtree MdbSargNode;
 
 typedef struct {
 	char *name;
-	unsigned char needs_length; /* or precision */
+	unsigned char needs_precision;
 	unsigned char needs_scale;
-	unsigned char needs_quotes;
+	unsigned char needs_byte_length;
+	unsigned char needs_char_length;
 } MdbBackendType;
 		
 typedef struct {
@@ -273,6 +276,8 @@ typedef struct {
 	guint16		tab_col_offset_var;
 	guint16		tab_col_offset_fixed;
 	guint16		tab_row_col_num_offset;
+	guint16		col_scale_offset;
+	guint16		col_prec_offset;
 } MdbFormatConstants; 
 
 typedef struct {
@@ -539,6 +544,10 @@ void mdb_set_boolean_fmt_words(MdbHandle *mdb);
 void mdb_set_boolean_fmt_numbers(MdbHandle *mdb);
 int mdb_read_row(MdbTableDef *table, unsigned int row);
 
+/* money.c */
+char *mdb_money_to_string(MdbHandle *mdb, int start);
+char *mdb_numeric_to_string(MdbHandle *mdb, int start, int scale, int prec);
+
 /* dump.c */
 void mdb_buffer_dump(const void *buf, off_t start, size_t len);
 
@@ -559,7 +568,7 @@ void mdb_register_backend(MdbHandle *mdb, char *backend_name, guint32 capabiliti
         const char *table_comment_statement, const char *per_table_comment_statement,
         gchar* (*quote_schema_name)(const gchar*, const gchar*));
 int  mdb_set_default_backend(MdbHandle *mdb, const char *backend_name);
-void mdb_print_schema(MdbHandle *mdb, FILE *outfile, char *tabname, char *dbnamespace, guint32 export_options);
+int  mdb_print_schema(MdbHandle *mdb, FILE *outfile, char *tabname, char *dbnamespace, guint32 export_options);
 void mdb_print_col(FILE *outfile, gchar *col_val, int quote_text, int col_type, int bin_len, char *quote_char, char *escape_char, int flags);
 
 /* sargs.c */
@@ -638,6 +647,8 @@ void mdb_iconv_init(MdbHandle *mdb);
 void mdb_iconv_close(MdbHandle *mdb);
 const char* mdb_target_charset(MdbHandle *mdb);
 
+/** @}*/
+
 #ifdef __cplusplus
   }
 #endif
