@@ -28,6 +28,10 @@
 #include <limits.h>
 #include "mdbodbc.h"
 
+/** \addtogroup odbc
+ *  @{
+ */
+
 //#define TRACE(x) fprintf(stderr,"Function %s\n", x);
 #define TRACE(x)
 
@@ -37,7 +41,6 @@ static int _odbc_fix_literals(struct _hstmt *stmt);
 //static int _odbc_get_server_type(int clt_type);
 static int _odbc_get_string_size(int size, SQLCHAR *str);
 
-static void bind_columns (struct _hstmt*);
 static void unbind_columns (struct _hstmt*);
 
 #define FILL_FIELD(f,v,s) mdb_fill_temp_field(f,v,s,0,0,0,0)
@@ -287,6 +290,7 @@ SQLRETURN SQL_API SQLExtendedFetch(
     SQLUSMALLINT      *rgfRowStatus)
 {
 	struct _hstmt *stmt = (struct _hstmt *) hstmt;
+	struct _sql_bind_info *cur = stmt->bind_head;
 
 	TRACE("SQLExtendedFetch");
 	if (fFetchType!=SQL_FETCH_NEXT) {
@@ -298,11 +302,21 @@ SQLRETURN SQL_API SQLExtendedFetch(
 	if (rgfRowStatus)
 		*rgfRowStatus = SQL_SUCCESS; /* what is the row status value? */
 	
-	bind_columns(stmt);
-
 	if (mdb_fetch_row(stmt->sql->cur_table)) {
+		SQLRETURN final_retval = SQL_SUCCESS;
+		while (cur && (final_retval == SQL_SUCCESS || final_retval == SQL_SUCCESS_WITH_INFO)) {
+			/* log error ? */
+			SQLLEN lenbind = 0;
+			SQLRETURN this_retval = SQLGetData(hstmt, cur->column_number, cur->column_bindtype,
+					cur->varaddr, cur->column_bindlen, &lenbind);
+			if (cur->column_lenbind)
+				*(cur->column_lenbind) = lenbind;
+			if (this_retval != SQL_SUCCESS)
+				final_retval = this_retval;
+			cur = cur->next;
+		}
 		stmt->rows_affected++;
-		return SQL_SUCCESS;
+		return final_retval;
 	} else {
 		return SQL_NO_DATA_FOUND;
 	}
@@ -1027,25 +1041,6 @@ SQLRETURN SQL_API SQLExecDirectW(
 }
 #endif // ENABLE_ODBC_W
 
-static void
-bind_columns(struct _hstmt *stmt)
-{
-	struct _sql_bind_info *cur;
-
-	TRACE("bind_columns");
-
-	if (stmt->rows_affected==0) {
-		cur = stmt->bind_head;
-		while (cur) {
-			if (mdb_sql_bind_column(stmt->sql, cur->column_number,
-						cur->varaddr, cur->column_lenbind) == -1) {
-				/* log error ? */
-			}
-			cur = cur->next;
-		}
-	}
-}
-
 static void
 unbind_columns(struct _hstmt *stmt)
 {
@@ -1067,26 +1062,28 @@ SQLRETURN SQL_API SQLFetch(
     SQLHSTMT           hstmt)
 {
 	struct _hstmt *stmt = (struct _hstmt *) hstmt;
+	struct _sql_bind_info *cur = stmt->bind_head;
 
 	TRACE("SQLFetch");
-	/* if we bound columns, transfer them to res_info now that we have one */
-	bind_columns(stmt);
-	//cur = stmt->bind_head;
-	//while (cur) {
-		//if (cur->column_number>0 &&
-			//cur->column_number <= stmt->sql->num_columns) {
-			// if (cur->column_lenbind) *(cur->column_lenbind) = 4;
-		//}
-		//cur = cur->next;
-	//}
 	if ( stmt->sql->limit >= 0 && stmt->rows_affected == stmt->sql->limit ) {
 		return SQL_NO_DATA_FOUND;
 	}
-
 	if (mdb_fetch_row(stmt->sql->cur_table)) {
+		SQLRETURN final_retval = SQL_SUCCESS;
+		while (cur && (final_retval == SQL_SUCCESS || final_retval == SQL_SUCCESS_WITH_INFO)) {
+			/* log error ? */
+			SQLLEN lenbind = 0;
+			SQLRETURN this_retval = SQLGetData(hstmt, cur->column_number, cur->column_bindtype,
+					cur->varaddr, cur->column_bindlen, &lenbind);
+			if (cur->column_lenbind)
+				*(cur->column_lenbind) = lenbind;
+			if (this_retval != SQL_SUCCESS)
+				final_retval = this_retval;
+			cur = cur->next;
+		}
 		stmt->rows_affected++;
 		stmt->pos=0;
-		return SQL_SUCCESS;
+		return final_retval;
 	} else {
 		return SQL_NO_DATA_FOUND;
 	}
@@ -1228,7 +1225,7 @@ SQLRETURN SQL_API SQLPrepare(
 
 	TRACE("SQLPrepare");
 
-	snprintf(stmt->query, sizeof(stmt->query), "%*s", sqllen, (char*)szSqlStr);
+	snprintf(stmt->query, sizeof(stmt->query), "%.*s", sqllen, (char*)szSqlStr);
 
 	return SQL_SUCCESS;
 }
@@ -1344,7 +1341,10 @@ SQLRETURN SQL_API SQLColumns(
 			LogStatementError(stmt, "Could not read table '%s'", szTableName);
 			return SQL_ERROR;
 		}
-		mdb_read_columns(table);
+		if (!mdb_read_columns(table)) {
+			LogStatementError(stmt, "Could not read columns of table '%s'", szTableName);
+			return SQL_ERROR;
+		}
 		for (j=0; j<table->num_cols; j++) {
 			col = g_ptr_array_index(table->columns, j);
 
@@ -1662,8 +1662,14 @@ SQLRETURN SQL_API SQLGetData(
 				strcpy(stmt->sqlState, "HY090"); // Invalid string or buffer length
 				return SQL_ERROR;
 			}
-			char *str = mdb_col_to_string(mdb, mdb->pg_buf,
-					col->cur_value_start, col->col_type, col->cur_value_len);
+			char *str = NULL;
+			if (col->col_type == MDB_NUMERIC) {
+				str = mdb_numeric_to_string(mdb, col->cur_value_start,
+						col->col_scale, col->col_prec);
+			} else {
+				str = mdb_col_to_string(mdb, mdb->pg_buf,
+						col->cur_value_start, col->col_type, col->cur_value_len);
+			}
 			size_t len = strlen(str);
 
 			if (stmt->pos >= len) {
@@ -2372,3 +2378,4 @@ static const char * _odbc_get_client_type_name(MdbColumn *col)
 	return NULL;
 }
 
+/** @}*/
