@@ -32,6 +32,10 @@
 
 #include <locale.h>
 
+/** \addtogroup mdbsql
+ *  @{
+ */
+
 /* Prevent warnings from -Wmissing-prototypes.  */
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
@@ -81,13 +85,13 @@ MdbSQL *sql = g_malloc0(sizeof(MdbSQL));
 #endif
 
 /**
- * mdb_sql_run_query:
- * @sql: MDB SQL object to execute the query on.
- * @querystr: SQL query string to execute.
  *
- * Parses @querystr and executes it within the given @sql object.
+ * @param sql: MdbSQL object to execute the query on.
+ * @param querystr: SQL query string to execute.
  *
- * Returns: the updated MDB SQL object, or NULL on error
+ * Parses \p querystr and executes it within the given \p sql object.
+ *
+ * @return  the updated MDB SQL object, or NULL on error
  **/
 MdbSQL*
 mdb_sql_run_query (MdbSQL* sql, const gchar* querystr) {
@@ -422,7 +426,7 @@ mdb_sql_eval_expr(MdbSQL *sql, char *const1, int op, char *const2)
 	MdbSargNode *node;
 
 	if (const1[0]=='\'' && const2[0]=='\'') {
-		value = strcmp(const1, const2);
+		value = strcoll(const1, const2);
 		switch (op) {
 			case MDB_EQUAL: compar = (value ? 0 : 1); break;
 			case MDB_GT: compar = (value > 0); break;
@@ -482,7 +486,7 @@ mdb_sql_add_sarg(MdbSQL *sql, char *col_name, int op, char *constant)
 	** column definition can be checked for validity
 	*/
 	if (constant[0]=='\'') {
-		snprintf(node->value.s, sizeof(node->value.s), "%*s", (int)strlen(constant) - 2, &constant[1]);
+		snprintf(node->value.s, sizeof(node->value.s), "%.*s", (int)strlen(constant) - 2, &constant[1]);
 		node->val_type = MDB_TEXT;
 	} else if ((p=strchr(constant, '.'))) {
 		*p=localeconv()->decimal_point[0];
@@ -696,7 +700,12 @@ void mdb_sql_describe_table(MdbSQL *sql)
 		return;
 	}
 
-	mdb_read_columns(table);
+	if (!mdb_read_columns(table)) {
+		mdb_sql_error(sql, "Could not read columns of table %s", sql_tab->name);
+		/* the column and table names are no good now */
+		mdb_sql_reset(sql);
+		return;
+    }
 
 	ttable = mdb_create_temp_table(mdb, "#describe");
 
@@ -794,7 +803,12 @@ int found = 0;
 		mdb_sql_reset(sql);
 		return;
 	}
-	mdb_read_columns(table);
+	if (!mdb_read_columns(table)) {
+		mdb_sql_error(sql, "Could not read columns of table %s", sql_tab->name);
+		/* the column and table names are no good now */
+		mdb_sql_reset(sql);
+		return;
+    }
 
 	if (sql->sel_count && !sql->sarg_tree) {
 		MdbTableDef *ttable = mdb_create_temp_table(mdb, "#count");
@@ -970,3 +984,4 @@ mdb_sql_dump_results(MdbSQL *sql)
 	/* the column and table names are no good now */
 	mdb_sql_reset(sql);
 }
+/** @}*/
