@@ -17,12 +17,14 @@
  */
 
 #include "mdbtools.h"
+#include "mdbver.h"
 
 #define EXPORT_BIND_SIZE 200000
 
 #define is_binary_type(x) (x==MDB_OLE || x==MDB_BINARY || x==MDB_REPID)
 
 static char *escapes(char *s);
+static void format_value(FILE *outfile, char *value, size_t length, int quote_text, int col_type, char *escape_char, char *quote_char, int bin_mode, int export_flags, char *backend_name);
 
 int
 main(int argc, char **argv)
@@ -54,6 +56,9 @@ main(int argc, char **argv)
 	char *value;
 	size_t length;
 	int ret;
+	char *locale = NULL;
+	char *table_name = NULL;
+	int print_mdbver = 0;
 
 	GOptionEntry entries[] = {
 		{"no-header", 'H', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &header_row, "Suppress header row.", NULL},
@@ -71,6 +76,7 @@ main(int argc, char **argv)
 		{"null", '0', 0, G_OPTION_ARG_STRING, &null_text, "Use <char> to represent a NULL value", "char"},
 		{"bin", 'b', 0, G_OPTION_ARG_STRING, &str_bin_mode, "Binary export mode", "strip|raw|octal|hex"},
 		{"boolean-words", 'B', 0, G_OPTION_ARG_NONE, &boolean_words, "Use TRUE/FALSE in Boolean fields (default is 0/1)", NULL},
+		{"version", 0, 0, G_OPTION_ARG_NONE, &print_mdbver, "Show mdbtools version and exit", NULL},
 		{NULL},
 	};
 	GError *error = NULL;
@@ -78,6 +84,7 @@ main(int argc, char **argv)
 
 	opt_context = g_option_context_new("<file> <table> - export data from MDB file");
 	g_option_context_add_main_entries(opt_context, entries, NULL /*i18n*/);
+	locale = setlocale(LC_CTYPE, "");
 	// g_option_context_set_strict_posix(opt_context, TRUE); /* options first, requires glib 2.44 */
 	if (!g_option_context_parse (opt_context, &argc, &argv, &error))
 	{
@@ -85,12 +92,24 @@ main(int argc, char **argv)
 		fputs(g_option_context_get_help(opt_context, TRUE, NULL), stderr);
 		exit (1);
 	}
-
+	if (print_mdbver) {
+		if (argc > 1) {
+			fputs(g_option_context_get_help(opt_context, TRUE, NULL), stderr);
+		}
+		fprintf(stdout,"%s\n", MDB_FULL_VERSION);
+		exit(argc > 1);
+	}
 	if (argc != 3) {
 		fputs("Wrong number of arguments.\n\n", stderr);
 		fputs(g_option_context_get_help(opt_context, TRUE, NULL), stderr);
 		exit(1);
 	}
+	table_name = g_locale_to_utf8(argv[2], -1, NULL, NULL, &error);
+	if (!table_name) {
+		fprintf(stderr, "argument parsing failed: %s\n", error->message);
+		exit(1);
+	}
+	setlocale(LC_CTYPE, locale);
 
 	/* Process options */
 	if (quote_char)
@@ -166,9 +185,9 @@ main(int argc, char **argv)
 			exit(1);
 		}
 
-	table = mdb_read_table_by_name(mdb, argv[2], MDB_TABLE);
+	table = mdb_read_table_by_name(mdb, table_name, MDB_TABLE);
 	if (!table) {
-		fprintf(stderr, "Error: Table %s does not exist in this database.\n", argv[2]);
+		fprintf(stderr, "Error: Table %s does not exist in this database.\n", table_name);
 		/* Don't bother clean up memory before exit */
 		exit(1);
 	}
@@ -206,7 +225,7 @@ main(int argc, char **argv)
 			if (counter % batch_size == 0) {
 				counter = 0; // reset to 0, prevent overflow on extremely large data sets.
 				char *quoted_name;
-				quoted_name = mdb->default_backend->quote_schema_name(namespace, argv[2]);
+				quoted_name = mdb->default_backend->quote_schema_name(namespace, table_name);
 				fprintf(outfile, "INSERT INTO %s (", quoted_name);
 				free(quoted_name);
 				for (i = 0; i < table->num_cols; i++) {
@@ -238,7 +257,11 @@ main(int argc, char **argv)
 						value = bound_values[i];
 						length = bound_lens[i];
 					}
-					mdb_print_col(outfile, value, quote_text, col->col_type, length, quote_char, escape_char, bin_mode | export_flags);
+					format_value(outfile, value, length,
+						     quote_text, col->col_type,
+						     escape_char, quote_char,
+						     bin_mode, export_flags,
+						     mdb->backend_name);
 					if (col->col_type == MDB_OLE)
 						free(value);
 				}
@@ -260,7 +283,7 @@ main(int argc, char **argv)
 
 			if (insert_dialect) {
 				char *quoted_name;
-				quoted_name = mdb->default_backend->quote_schema_name(namespace, argv[2]);
+				quoted_name = mdb->default_backend->quote_schema_name(namespace, table_name);
 				fprintf(outfile, "INSERT INTO %s (", quoted_name);
 				free(quoted_name);
 				for (i = 0; i < table->num_cols; i++) {
@@ -290,25 +313,11 @@ main(int argc, char **argv)
 						value = bound_values[i];
 						length = bound_lens[i];
 					}
-					/* Correctly handle insertion of binary blobs into SQLite using the string literal notation of X'1234ABCD...' */
-					if (!strcmp(mdb->backend_name, "sqlite") && is_binary_type(col->col_type)
-							&& bin_mode == MDB_BINEXPORT_HEXADECIMAL) {
-						char *quote_char_binary_sqlite = (char *) g_strdup("'");
-						fputs("X", outfile);
-						mdb_print_col(outfile, value, quote_text, col->col_type, length, quote_char_binary_sqlite, escape_char, bin_mode | export_flags);
-						g_free (quote_char_binary_sqlite);
-						/* Correctly handle insertion of binary blobs into PostgreSQL using the notation of decode('1234ABCD...', 'hex') */
-					} else if (!strcmp(mdb->backend_name, "postgres") && is_binary_type(col->col_type)
-							&& bin_mode == MDB_BINEXPORT_HEXADECIMAL) {
-						char *quote_char_binary_postgres = (char *) g_strdup("'");
-						fputs("decode(", outfile);
-						mdb_print_col(outfile, value, quote_text, col->col_type, length, quote_char_binary_postgres, escape_char, bin_mode | export_flags);
-						fputs(", 'hex')", outfile);
-						g_free (quote_char_binary_postgres);
-						/* No special treatment for other backends or when hexadecimal notation hasn't been selected with the -b hex command line option */
-					} else {
-						mdb_print_col(outfile, value, quote_text, col->col_type, length, quote_char, escape_char, bin_mode | export_flags);
-					}
+					format_value(outfile, value, length,
+						     quote_text, col->col_type,
+						     escape_char, quote_char,
+						     bin_mode, export_flags,
+						     mdb->backend_name);
 					if (col->col_type == MDB_OLE)
 						free(value);
 				}
@@ -338,9 +347,43 @@ main(int argc, char **argv)
 	g_free(escape_char);
 	g_free(namespace);
 	g_free(str_bin_mode);
+	g_free(table_name);
 	return 0;
 }
 
+static void format_value(FILE *outfile, char *value, size_t length, int quote_text, int col_type, char *escape_char, char *quote_char, int bin_mode, int export_flags, char *backend_name)
+{
+	/* Correctly handle insertion of binary blobs into sqlite3 using the notation of X'1234ABCD...') */
+	if (!strcmp(backend_name, "sqlite")
+			&& is_binary_type(col_type)
+			&& bin_mode == MDB_BINEXPORT_HEXADECIMAL) {
+		char *quote_char_binary_sqlite = (char *) g_strdup("'");
+		fputs("X", outfile);
+		mdb_print_col(outfile, value, quote_text, col_type, length, quote_char_binary_sqlite, escape_char, bin_mode | export_flags);
+		g_free (quote_char_binary_sqlite);
+		/* Correctly handle insertion of binary blobs into MySQL using the notation of 0x1234ABCD...) */
+	} else if (!strcmp(backend_name, "mysql")
+			&& is_binary_type(col_type)
+			&& bin_mode == MDB_BINEXPORT_HEXADECIMAL) {
+		char *quote_char_binary_sqlite = (char *) g_strdup("");
+		fputs("0x", outfile);
+		mdb_print_col(outfile, value, quote_text, col_type, length, quote_char_binary_sqlite, escape_char, bin_mode | export_flags);
+		g_free (quote_char_binary_sqlite);
+		/* Correctly handle insertion of binary blobs into PostgreSQL using the notation of decode('1234ABCD...', 'hex') */
+	} else if (!strcmp(backend_name, "postgres")
+			&& is_binary_type(col_type)
+			&& bin_mode == MDB_BINEXPORT_HEXADECIMAL) {
+		char *quote_char_binary_postgres = (char *) g_strdup("'");
+		fputs("decode(", outfile);
+		mdb_print_col(outfile, value, quote_text, col_type, length, quote_char_binary_postgres, escape_char, bin_mode | export_flags);
+		fputs(", 'hex')", outfile);
+		g_free (quote_char_binary_postgres);
+		/* No special treatment for other backends or when hexadecimal notation hasn't been selected with the -b hex command line option */
+	} else {
+		mdb_print_col(outfile, value, quote_text, col_type, length, quote_char, escape_char, bin_mode | export_flags);
+	}
+}
+
 static char *escapes(char *s)
 {
 	char *d = (char *) g_strdup(s);
